@@ -22,7 +22,6 @@ use crate::{
 /// # See also
 /// [`SyntaxNode`]
 /// [`SyntaxNode::new_root_with_resolver`]
-#[derive(Clone)]
 #[repr(transparent)]
 pub struct ResolvedNode<L: Language, D: 'static = ()> {
     pub(super) syntax: SyntaxNode<L, D>,
@@ -41,6 +40,14 @@ impl<L: Language, D> ResolvedNode<L, D> {
     }
 }
 
+impl<L: Language, D> Clone for ResolvedNode<L, D> {
+    fn clone(&self) -> Self {
+        Self {
+            syntax: self.syntax.clone(),
+        }
+    }
+}
+
 impl<L: Language, D> Deref for ResolvedNode<L, D> {
     type Target = SyntaxNode<L, D>;
 
@@ -77,6 +84,14 @@ impl<L: Language, D> ResolvedToken<L, D> {
     }
 }
 
+impl<L: Language, D> Clone for ResolvedToken<L, D> {
+    fn clone(&self) -> Self {
+        Self {
+            syntax: self.syntax.clone(),
+        }
+    }
+}
+
 impl<L: Language, D> Deref for ResolvedToken<L, D> {
     type Target = SyntaxToken<L, D>;
 
